#include "predef.h"
#include <stdio.h> 
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h> 
#include <dhcpclient.h>
#include <smarttrap.h>
#include <taskmon.h>
#include <NetworkDebug.h>
#include <cfinter.h>
#include <ip.h>
#include <udp.h>
#include <tcp.h>
#include <bsp.h>
#include <buffers.h>
#include <ethernet.h>
#include "StopWatch.h"
#include <constants.h>
#include <stdlib.h>

//----------------------------------------------------------------------------------------
// The following address of a local PC is required for the UDP test to work properly
#define MY_PC_IP_ADDRESS "10.1.1.190"
//---------------------------------------------------------------------------------------

#if (defined MOD5270 || defined PK70 || defined SB70 )
#include "sim5270.h"
#elif (defined MOD5234 || defined CB34EX)
#include "sim5234.h"
#elif (defined MOD5272 || defined SB72 || defined SB72EX ) 
#include "sim5272.h"
#elif (defined MOD5282) 
#include "sim5282.h"
#elif ( defined MCF5208 ) 
#include "sim5208.h"
#endif 

#define LOGME iprintf("We got to line %d in file %s\r\n",__LINE__ , __FILE__ );

const char * AppName="NB Benchmark";

extern "C" {
	void NMI_C_Part();
void NMI_ASM_Part(); // The part of the ISR written in Assembly
#if (defined MOD5272 || defined SB72 || defined SB72EX || defined MCF5208 ) 
#elif ( defined MOD5282 || defined MOD5234 || defined CB34EX )
void SetIntc( int intc, long func, int vector, int level, int prio );
#else
void SetIntc( long func, int vector, int level, int prio );
#endif
void UserMain(void * pd);
}

#ifndef ENABLE_SRAM_SYS
#define FAST_USER_VAR
#define FAST_USER_STK
#endif

void PrintSRAMUsage();
//
// Task Switch Variables
//
void MyTask( void *pdata );
double PendTest( DWORD Pends ); 
#define PEND_LIMIT (100000) 
#define MY_TASK_STACK_SIZE (256)
DWORD MyTaskStack[MY_TASK_STACK_SIZE]__attribute__((aligned( 4 ))) FAST_USER_STK;
OS_SEM MySemaphore FAST_USER_VAR;
double TotalTime __attribute__((aligned( 8 ))) FAST_USER_VAR;

// 
// IRQ Switch Variables
// 
#define TOTAL_ISRS (1000000)
#define TOTAL_NM_ISRS (1000000)

#if (!defined MOD5272 || !defined SB72 || !defined SB72EX ) 
volatile DWORD ISRcount __attribute__((aligned( 4 ))) FAST_USER_VAR; 
#endif
// 
// UDP Upload Variables
//
#define portnum (0x1234)
IPADDR ipaddr FAST_USER_VAR;
void SendPacket();
void UDPTest();
volatile int Pleft FAST_USER_VAR; 
//char tempppp[1000] USER_VAR_ATTR;
// 
// TCP Upload Variables
//
#define BUFFER_SIZE 2048
char DataBuffer[BUFFER_SIZE]__attribute__((aligned( 16 )));
extern DWORD taskidle;
char input[16];
void TCPTest();
/*----------------------------------------------------------------------------------*/   
#if (!defined MOD5272 || !defined SB72 || !defined SB72EX || !defined MCF5208 ) 
INTERRUPT( IRQ5_ISR, 0x2600 )
{
	ISRcount ++;
	sim.eport.epfr = 0x20;   // Clear any pending irq flag
}
INTERRUPT( IRQ7_ISR, 0x2700 ){}

/*----------------------------------------------------------------------------------*/   

void Function_Holder()
{
   __asm__  (" .global  NMI_ASM_Part"); // export the label for ASM part
   __asm__  (" .extern  NMI_C_Part");   // label for the C part of the ISR
   __asm__  ("NMI_ASM_Part:");          // label for the assembly part of the ISR
  // __asm__  (" move.w #0x2700,%sr ");   // set the IRQ mask to mask all 
	__asm__  (" addq.l  #1, (ISRcount) "); // 
	__asm__  (" move.l %d0, %a7@- "); // 
	__asm__  (" move.l #0x80, %d0 "); // 
	__asm__  (" move.b %d0, 0x40130006 "); // 
	__asm__  (" move.l %a7@+, %d0 ");
   __asm__  (" rte");
}
#endif
/*----------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------------*/
#ifdef RamTestEnable
#define RamTestSize (256 * 1024 * 2 )
DWORD RamTest[RamTestSize];
#endif

void UserMain(void * pd) {
    InitializeStack();
    if (EthernetIP == 0) GetDHCPAddress();
    OSChangePrio(MAIN_PRIO);
    EnableAutoUpdate();
    EnableTaskMonitor();

    #ifndef _DEBUG
    EnableSmartTraps();
    #endif

    #ifdef _DEBUG
    InitializeNetworkGDB_and_Wait();
    #endif

    iprintf("%s application started\r\n\r\n", AppName );

    for(int i =1; i< BUFFER_SIZE; i++)
    	    DataBuffer[i-1] = i;
        	       	    
	 PrintSRAMUsage();
	 
    InitStopWatch();
       
#ifdef RamTestEnable
    StartStopWatch();
    for(volatile int i = 0; i<16; i++){
    memset( RamTest, i, (RamTestSize*4) );
    memcpy( (BYTE*)RamTest+(RamTestSize*2), RamTest, (RamTestSize*2) );
}

    TotalTime = ReadTime();
        StopClearStopWatch();
        printf("RAM Test Took %g Seconds\n\r",TotalTime);
#endif

    OSSemInit( &MySemaphore, 0 );
	 if ( OSTaskCreate( MyTask,
                      NULL,
                      ( void * ) &MyTaskStack[MY_TASK_STACK_SIZE],
                      ( void * ) MyTaskStack,
                      MAIN_PRIO+1 ) != OS_NO_ERR )
  	 {
      iprintf( "*** Error creating task\r\n" );
  	 }
    
    // OPEN THE DUAL PORT SRAM
    // This can increase Ethernet throughput but also Task Switch and IRQ Switching Time 
    //sim.scm.rambar = 0x20000200;
    //asm("movel #0x20000201, %d0"); 	
	 //asm("movec %d0, #0x0C05"); 
/*----------------------------------------------------------------------------------*/    
    printf("\r\nStarting Task Switch Latency Test\r\n");
    PendTest( 1 );
    PendTest( PEND_LIMIT ); //about 40 seconds
    printf("%d Pends took %g Seconds or %g Seconds Per Pend\n\r", PEND_LIMIT, TotalTime, TotalTime/(double)PEND_LIMIT);
    printf("Each pend is 2 OS Task Switches which = %g seconds per switch\n\r\r\n", TotalTime/((double)PEND_LIMIT * (double)2));
 
 /*----------------------------------------------------------------------------------*/    
 #if(!defined MOD5272 && !defined SB72 && !defined SB72EX && !defined MCF5208 ) 
    printf("Starting Standard IRQ Latency Test\r\n");
    sim.eport.epddr |= 0x20;  //sets IRQ5 in direction register to be an output
    sim.eport.epdr &= ~0x20;  //drive IRQ5 low
    sim.eport.eppar |= 0xC00;  //set to trigger on any edge
    sim.eport.epfr |= 0x20;   // Clear any pending irq flag
    sim.eport.epier |= 0x20;  //enable IRQ5 interrupts
    #if( defined MOD5282 || defined MOD5234 || defined CB34EX )
    SetIntc( 0 , ( long )&IRQ5_ISR, 5, 5, 5 );   // initialize IRQ5 to use SRAM ISR 
    #else
    SetIntc( ( long )&IRQ5_ISR, 5, 5, 5 );   // initialize IRQ5 to use SRAM ISR 
    #endif
    ISRcount = 0;
    StartStopWatch();   
    while( ISRcount < TOTAL_ISRS )
		sim.eport.epdr ^= 0x20;  //drive IRQ5 high
    TotalTime = ReadTime();
    StopClearStopWatch();
    sim.eport.epier &= ~0x20;  //disable IRQ5 interrupts
    printf("%d interrupts took %g Seconds or %g Seconds Per interrupt\n\r", TOTAL_ISRS, TotalTime, TotalTime/(double)TOTAL_ISRS);
    printf("Each IRQ has 2 context switches which = %g seconds per IRQ switch\n\r\r\n", TotalTime/((double)TOTAL_ISRS * (double)2));

/*----------------------------------------------------------------------------------*/

   printf("Starting Non-Maskable IRQ Latency Test\r\n");
    sim.eport.epddr |= 0x80;  //sets IRQ7 in direction register to be an output
    sim.eport.epdr &= ~0x80;  //drive IRQ7 low
    sim.eport.eppar |= 0xC000;  //set to trigger on any edge
    sim.eport.epfr |= 0x80;   // Clear any pending irq flag
    sim.eport.epier |= 0x80;  //enable IRQ7 interrupts
    #if ( defined MOD5282 || defined MOD5234 || defined CB34EX )
    SetIntc( 0 , ( long )&NMI_ASM_Part, 7, 7, 7 );   // initialize IRQ7 to use SRAM ISR 
    #else
    SetIntc( ( long )&NMI_ASM_Part, 7, 7, 7 );   // initialize IRQ7 to use SRAM ISR 
    #endif
    ISRcount = 0;
    StartStopWatch();    
    while( ISRcount <  TOTAL_NM_ISRS )
	 	sim.eport.epdr ^= 0x80;
    TotalTime = ReadTime();
    StopClearStopWatch();
    sim.eport.epier &= ~0x80;  //disable IRQ5 interrupts
    printf("%d interrupts took %g Seconds or %g Seconds Per Non-Maskable Interrupt\n\r", TOTAL_ISRS, TotalTime, TotalTime/(double)TOTAL_ISRS);
    printf("Each IRQ has 2 context switches which = %g seconds per Non-Maskable IRQ switch\n\r\r\n", TotalTime/((double)TOTAL_ISRS * (double)2));
#endif
/*----------------------------------------------------------------------------------*/ 

	 UDPTest();
	 	
/*----------------------------------------------------------------------------------*/ 
	
 	printf("Starting TCP Upload Data Payload Test\r\n");
	printf("Type \"speed <moduleIP>\" at the PC command prompt to start\r\n");
   register DWORD dw1,dw2;
   register DWORD nsent=0;
   register int rv;
   for (int i=0; i< BUFFER_SIZE; i++) DataBuffer[i]=('A'+ (i % 64));
   int fdListen=listen(INADDR_ANY,1234,5);
	if (fdListen>0)
	{
	 iprintf("FD listen is ok\r\n");	
	 while(1)
	  {
	  if(charavail())
	  	{		gets(input);	UDPTest();}
	  int fda= accept(fdListen,NULL,NULL,5);
	  if (fda>1)
	    {
	     iprintf("Accepted Waiting\r\n");
	     setsockoption(fda,SO_NOPUSH); 
	     setsocketackbuffers(fda,40); 
	     read(fda,DataBuffer,1);
		  nsent=0;
	     dw1=taskidle;
	     StartStopWatch();   
	     while(1)
	      {
			 rv=write(fda,DataBuffer,TCP_BUFFER_SEGMENTS*1460);

	       if (rv<0) 
	        {
	         dw2=taskidle;
			 close(fda);
			 break;
		   }
		   else
			nsent+=rv;
	      }
	      TotalTime = ReadTime();
    		StopClearStopWatch();
    		printf("%gMByte of TCP data upload took %g seconds or %gMByte/s\n\r\r\n", 
    		(double)(nsent)/(double)(1024*1024), TotalTime, 
    		(double)(1/TotalTime) * (double)((double)(nsent)/(double)(1024*1024)));
	    }
	    else if (fda == 0)
	    iprintf("Accepfailed\r\n");
	  }
	 }
	else
	  iprintf("FD listen is NOT ok\r\n");
	
	printf("Press any key to go again\r\n"); 
	gets(input);
}


/*-------------------------------------------------------------------
 MyTask
 This task will post to a semaphore
 -------------------------------------------------------------------*/
void MyTask( void *pdata )
{
	while(1)
		OSSemPost( &MySemaphore );
}
double PendTest( DWORD Pends )  
{
   StartStopWatch(); 
   while( Pends )
   {
      OSSemPend( &MySemaphore, 0 );
      Pends--;
	}
   TotalTime = ReadTime();
   StopClearStopWatch(); 
   return TotalTime;
}

void UDPTest()

{
	ipaddr = AsciiToIp( MY_PC_IP_ADDRESS );
	 #ifndef _DEBUG
	 while( !EtherLink());
    #else
    printf("We are in debug mode\r\n");
    #endif
    {
    	 {
       //	 UDPPacket pkt;
	    //	 pkt.SetSourcePort( portnum );
	   // 	 pkt.SetDestinationPort( portnum );
	   // 	 pkt.AddData( (BYTE*)DataBuffer, (MAX_UDPDATA) );
	   // 	 pkt.Send( ipaddr );
       }
       printf("Starting UDP Upload Throughput Test\r\n");
       
       //Packet = 1500 Bytes, 10MBytes =  10485760 Bytes = 6990.5 packets
       StartStopWatch();  
       #define TotalPackets (6990)
      {
      UDPPacket pkt;
		pkt.SetSourcePort( portnum );
		pkt.SetDestinationPort( portnum );
		pkt.AddData( (BYTE*)DataBuffer, (MAX_UDPDATA) );
      for( Pleft = 0; Pleft < (TotalPackets-1); Pleft++ )
       {
       	// while(GetFreeCount() < 20 ){};
			//{
				//UDPPacket pkt;
			   //pkt.SetSourcePort( portnum );
			   //pkt.SetDestinationPort( portnum );
			   //pkt.AddData( (BYTE*)DataBuffer, (MAX_UDPDATA) );
			   pkt.SendAndKeep( ipaddr );
			//}
		 }
      pkt.Send( ipaddr );
      }
      UDPPacket pkt;
   	pkt.SetSourcePort( portnum );
    	pkt.SetDestinationPort( portnum );
    	pkt.AddData( (BYTE*)DataBuffer, (MAX_UDPDATA/2) );
    	pkt.Send( ipaddr );
    }
    TotalTime = ReadTime();
    StopClearStopWatch();
    printf("10MByte UDP upload took %g seconds or %gMByte/s\n\r\r\n", TotalTime, (double)(1/TotalTime) * (double)10);
}





void PrintSRAMUsage( ) 
{
#ifdef ENABLE_SRAM_SYS
	 extern BYTE SRAM_START, SRAM_END,
     				 SYS_VAR_SECT_END, USER_VAR_SECT_END, IDLE_STK_SECT_END, MAIN_STK_SECT_END,
    				 ETHER_STK_SECT_END, IP_STK_SECT_END, TCP_STK_SECT_END, HTTP_STK_SECT_END, 
    				 FTP_STK_SECT_END, WIFI_STK_SECT_END, PPP_STK_SECT_END, COMMAND_STK_SECT_END, 
    				 USER_STK_SECT_END, BUFFERS_VAR_SECT_END, ETHER_VAR_SECT_END, IP_VAR_SECT_END,
    				 TCP_VAR_SECT_END;
    printf("You are using %u bytes in SRAM\r\n", (unsigned int)&SRAM_END - (unsigned int)&SRAM_START);
    printf("SYS_VAR_SECT = %u bytes\r\n", 	(unsigned int)&SYS_VAR_SECT_END - (unsigned int)&SRAM_START);
    printf("IDLE_STK_SECT = %u bytes\r\n", 	(unsigned int)&IDLE_STK_SECT_END - (unsigned int)&SYS_VAR_SECT_END);
	 printf("MAIN_STK_SECT = %u bytes\r\n", 	(unsigned int)&MAIN_STK_SECT_END - (unsigned int)&IDLE_STK_SECT_END);
	 printf("ETHER_VAR_SECT = %u bytes\r\n", 	(unsigned int)&ETHER_VAR_SECT_END - (unsigned int)&MAIN_STK_SECT_END);
	 printf("ETHER_STK_SECT = %u bytes\r\n", 	(unsigned int)&ETHER_STK_SECT_END - (unsigned int)&ETHER_VAR_SECT_END);
	 printf("IP_VAR_SECT = %u bytes\r\n", 		(unsigned int)&IP_VAR_SECT_END - (unsigned int)&ETHER_STK_SECT_END);		
	 printf("IP_STK_SECT = %u bytes\r\n", 		(unsigned int)&IP_STK_SECT_END - (unsigned int)&IP_VAR_SECT_END);
	 printf("TCP_VAR_SECT = %u bytes\r\n", 	(unsigned int)&TCP_VAR_SECT_END - (unsigned int)&IP_STK_SECT_END);
	 printf("TCP_STK_SECT = %u bytes\r\n", 	(unsigned int)&TCP_STK_SECT_END - (unsigned int)&TCP_VAR_SECT_END);
	 printf("HTTP_STK_SECT = %u bytes\r\n", 	(unsigned int)&HTTP_STK_SECT_END - (unsigned int)&TCP_STK_SECT_END);
    printf("FTP_STK_SECT = %u bytes\r\n",  	(unsigned int)&FTP_STK_SECT_END - (unsigned int)&HTTP_STK_SECT_END);
	 printf("WIFI_STK_SECT = %u bytes\r\n", 	(unsigned int)&WIFI_STK_SECT_END - (unsigned int)&FTP_STK_SECT_END);
	 printf("PPP_STK_SECT = %u bytes\r\n", 	(unsigned int)&PPP_STK_SECT_END - (unsigned int)&WIFI_STK_SECT_END);
	 printf("COMMAND_STK_SECT = %u bytes\r\n",(unsigned int)&COMMAND_STK_SECT_END - (unsigned int)&PPP_STK_SECT_END);
	 printf("USER_VAR_SECT = %u bytes\r\n", 	(unsigned int)&USER_VAR_SECT_END - (unsigned int)&COMMAND_STK_SECT_END);
	 printf("USER_STK_SECT = %u bytes\r\n", 	(unsigned int)&USER_STK_SECT_END - (unsigned int)&USER_VAR_SECT_END);
	 printf("BUFF_VAR_SECT = %u bytes\r\n", 	(unsigned int)&BUFFERS_VAR_SECT_END - (unsigned int)&USER_STK_SECT_END);
#ifdef FAST_BUFFERS
    extern unsigned int FAST_BUFFER_POOL_SIZE;  
	 printf("There are %u fast buffers in the SRAM pool\r\n", FAST_BUFFER_POOL_SIZE);
	 extern buffer_list free_fast_list;
	  printf("There are %u fast buffers free (available)\r\n", free_fast_list.GetCount());
#endif
#endif
}

